"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _clientDynamodb = require("@aws-sdk/client-dynamodb");
var _libDynamodb = require("@aws-sdk/lib-dynamodb");
var _bluebird = _interopRequireDefault(require("bluebird"));
var _highland = _interopRequireDefault(require("highland"));
var _nodeHttpHandler = require("@smithy/node-http-handler");
var _utilRetry = require("@smithy/util-retry");
var _lodash = require("lodash");
var _retry = require("../utils/retry");
var _log = require("../utils/log");
/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */

class Connector {
  constructor({
    debug,
    tableName,
    convertEmptyValues,
    pipelineId,
    removeUndefinedValues = true,
    timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    retryConfig = _retry.defaultRetryConfig,
    additionalClientOpts = {},
    ...opt
  }) {
    this.debug = msg => debug('%j', msg);
    this.tableName = tableName || /* istanbul ignore next */'undefined';
    this.client = Connector.getClient(pipelineId, debug, convertEmptyValues, removeUndefinedValues, timeout, additionalClientOpts);
    this.retryConfig = retryConfig;
    this.opt = opt;
  }
  static clients = {};
  static getClient(pipelineId, debug, convertEmptyValues, removeUndefinedValues, timeout, additionalClientOpts) {
    const addlRequestHandlerOpts = (0, _lodash.pick)(additionalClientOpts, ['requestHandler']);
    const addlClientOpts = (0, _lodash.omit)(additionalClientOpts, ['requestHandler']);
    if (!this.clients[pipelineId]) {
      const dynamoClient = new _clientDynamodb.DynamoDBClient({
        requestHandler: new _nodeHttpHandler.NodeHttpHandler({
          requestTimeout: timeout,
          connectionTimeout: timeout,
          ...addlRequestHandlerOpts
        }),
        retryStrategy: new _utilRetry.ConfiguredRetryStrategy(11, _retry.defaultBackoffDelay),
        logger: (0, _log.defaultDebugLogger)(debug),
        ...addlClientOpts
      });
      this.clients[pipelineId] = _libDynamodb.DynamoDBDocumentClient.from(dynamoClient, {
        marshallOptions: {
          convertEmptyValues,
          removeUndefinedValues
        }
      });
    }
    return this.clients[pipelineId];
  }
  update(inputParams, ctx) {
    const params = {
      TableName: this.tableName,
      ...inputParams
    };
    return this._sendCommand(new _libDynamodb.UpdateCommand(params), ctx).catch(err => {
      /* istanbul ignore else */
      if (err.name === 'ConditionalCheckFailedException') {
        return {};
      }
      /* istanbul ignore next */
      return _bluebird.default.reject(err);
    });
  }
  put(inputParams, ctx) {
    const params = {
      TableName: this.tableName,
      ...inputParams
    };
    return this._sendCommand(new _libDynamodb.PutCommand(params), ctx);
  }
  batchGet(inputParams, ctx) {
    const params = {
      ...inputParams
    };
    return this._batchGet(params, [], ctx);
  }
  query(inputParams, ctx) {
    return this.queryAll(inputParams, ctx);
  }
  queryAll(inputParams, ctx) {
    const params = {
      TableName: this.tableName,
      ...inputParams
    };
    let cursor;
    let itemsCount = 0;
    return (0, _highland.default)((push, next) => {
      params.ExclusiveStartKey = cursor;
      return this._sendCommand(new _libDynamodb.QueryCommand(params), ctx).then(data => {
        itemsCount += data.Items.length;
        if (data.LastEvaluatedKey && (!params.Limit || params.Limit && itemsCount < params.Limit)) {
          cursor = data.LastEvaluatedKey;
        } else {
          cursor = undefined;
        }
        data.Items.forEach(obj => {
          push(null, obj);
        });
      }).catch( /* istanbul ignore next */err => {
        push(err, null);
      }).finally(() => {
        if (cursor) {
          next();
        } else {
          push(null, _highland.default.nil);
        }
      });
    }).collect().toPromise(_bluebird.default);
  }
  queryPage(inputParams, ctx) {
    const params = {
      TableName: this.tableName,
      ...inputParams
    };
    return this._sendCommand(new _libDynamodb.QueryCommand(params), ctx);
  }
  scan(inputParams, ctx) {
    const params = {
      TableName: this.tableName,
      ...inputParams
    };
    return this._sendCommand(new _libDynamodb.ScanCommand(params), ctx);
  }
  _batchGet(params, attempts, ctx) {
    (0, _retry.assertMaxRetries)(attempts, this.retryConfig.maxRetries);
    return (0, _retry.wait)((0, _retry.getDelay)(this.retryConfig.retryWait, attempts.length)).then(() => this._sendCommand(new _libDynamodb.BatchGetCommand(params), ctx).then(resp => {
      const response = {
        Responses: {},
        ...resp
      };
      if (Object.keys(response.UnprocessedKeys || /* istanbul ignore next */{}).length > 0) {
        return this._batchGet(unprocessed(params, response), [...attempts, response], ctx);
      } else {
        return accumlate(attempts, response);
      }
    }));
  }
  _sendCommand(command, ctx) {
    this.opt.metrics?.capture(this.client, command, 'dynamodb', this.opt, ctx);
    return _bluebird.default.resolve(this.client.send(command)).tap(this.debug).tapCatch(this.debug);
  }
}
var _default = exports.default = Connector;
const unprocessed = (params, resp) => ({
  ...params,
  RequestItems: resp.UnprocessedKeys
});
const accumlate = (attempts, resp) => attempts.reduceRight((a, c) => ({
  ...a,
  Responses: Object.keys(a.Responses).reduce((a2, c2) => ({
    ...a2,
    [c2]: [...(a2[c2] || []), ...a.Responses[c2]]
  }), {
    ...c.Responses
  }),
  attempts: [...attempts, resp]
}), resp);