"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _bluebird = _interopRequireDefault(require("bluebird"));
var _clientSqs = require("@aws-sdk/client-sqs");
var _nodeHttpHandler = require("@smithy/node-http-handler");
var _utilRetry = require("@smithy/util-retry");
var _lodash = require("lodash");
var _retry = require("../utils/retry");
var _log = require("../utils/log");
/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */

class Connector {
  constructor({
    debug,
    pipelineId,
    queueUrl = process.env.QUEUE_URL,
    timeout = Number(process.env.SQS_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    retryConfig = _retry.defaultRetryConfig,
    additionalClientOpts = {},
    ...opt
  }) {
    this.debug = msg => debug('%j', msg);
    this.queueUrl = queueUrl || 'undefined';
    this.client = Connector.getClient(pipelineId, debug, timeout, additionalClientOpts);
    this.retryConfig = retryConfig;
    this.opt = opt;
  }
  static clients = {};
  static getClient(pipelineId, debug, timeout, additionalClientOpts) {
    const addlRequestHandlerOpts = (0, _lodash.pick)(additionalClientOpts, ['requestHandler']);
    const addlClientOpts = (0, _lodash.omit)(additionalClientOpts, ['requestHandler']);
    if (!this.clients[pipelineId]) {
      this.clients[pipelineId] = new _clientSqs.SQSClient({
        requestHandler: new _nodeHttpHandler.NodeHttpHandler({
          requestTimeout: timeout,
          connectionTimeout: timeout,
          ...addlRequestHandlerOpts
        }),
        retryStrategy: new _utilRetry.ConfiguredRetryStrategy(11, _retry.defaultBackoffDelay),
        logger: (0, _log.defaultDebugLogger)(debug),
        ...addlClientOpts
      });
    }
    return this.clients[pipelineId];
  }
  sendMessageBatch(inputParams, ctx) {
    const params = {
      QueueUrl: this.queueUrl,
      ...inputParams
    };
    return this._sendMessageBatch(params, [], ctx);
  }
  _sendMessageBatch(params, attempts, ctx) {
    (0, _retry.assertMaxRetries)(attempts, this.retryConfig.maxRetries);
    return (0, _retry.wait)((0, _retry.getDelay)(this.retryConfig.retryWait, attempts.length)).then(() => this._sendCommand(new _clientSqs.SendMessageBatchCommand(params), ctx).tap(this.debug).tapCatch(this.debug).then(resp => {
      if (resp.Failed?.length > 0) {
        return this._sendMessageBatch(unprocessed(params, resp), [...attempts, resp]);
      } else {
        return accumlate(attempts, resp);
      }
    }));
  }
  _sendCommand(command, ctx) {
    this.opt.metrics?.capture(this.client, command, 'sqs', this.opt, ctx);
    return _bluebird.default.resolve(this.client.send(command)).tap(this.debug).tapCatch(this.debug);
  }
}
var _default = exports.default = Connector;
const unprocessed = (params, resp) => ({
  ...params,
  Entries: resp.Failed.map(m => params.Entries.find(e => m.Id === e.Id))
});
const accumlate = (attempts, resp) => attempts.reduceRight((a, c) => ({
  ...a,
  Successful: [...c.Successful, ...a.Successful],
  attempts: [...attempts, resp]
}), resp);