"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toDynamodbRecords = exports.outTtlExpiredEvents = exports.outReplicas = exports.outGlobalTableExtraModify = exports.fromDynamodb = exports.UNKNOWN_DYNAMODB_EVENT_TYPE = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _utilDynamodb = require("@aws-sdk/util-dynamodb");
var _utils = require("../utils");
/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */

const fromDynamodb = (event, {
  pkFn = 'pk',
  skFn = 'sk',
  discriminatorFn = 'discriminator',
  eventTypePrefix = undefined,
  ignoreTtlExpiredEvents = false,
  ignoreReplicas = true
} = {}) =>
// eslint-disable-line import/prefer-default-export

// prepare the event stream
(0, _highland.default)(event.Records)

//--------------------------------
// global table support
.filter(outReplicas(ignoreReplicas)).filter(outGlobalTableExtraModify)
//--------------------------------
// ttl support
.filter(outTtlExpiredEvents(ignoreTtlExpiredEvents))
//--------------------------------
.map((0, _utils.faulty)(record => (
// create a unit-of-work for each event
// so we can correlate related work for error handling
{
  record,
  event: {
    id: record.eventID,
    type: `${calculateEventTypePrefix(record, {
      skFn,
      discriminatorFn,
      eventTypePrefix
    })}-${calculateEventTypeSuffix(record)}`,
    partitionKey: record.dynamodb.Keys[pkFn].S,
    timestamp: record.dynamodb.ApproximateCreationDateTime * 1000,
    tags: {
      region: record.awsRegion
    },
    raw: {
      new: record.dynamodb.NewImage ? (0, _utilDynamodb.unmarshall)(record.dynamodb.NewImage) : undefined,
      old: record.dynamodb.OldImage ? (0, _utilDynamodb.unmarshall)(record.dynamodb.OldImage) : undefined
    }
  }
}))).tap(uow => (0, _utils.options)().metrics?.adornDynamoMetrics(uow, event));

// https://www.trek10.com/blog/dynamodb-single-table-relational-modeling
// all rows must have a discriminator field to store the prefix: <entityname> or <entityname-relationship>
// for node rows this could be the same value as the sk
// for edge rows the sk is used as the fk value
// so if the table includes edge rows then the discriminator field should hold the prefix
exports.fromDynamodb = fromDynamodb;
const calculateEventTypePrefix = (record, opt) => {
  /* istanbul ignore if */if (opt.eventTypePrefix) return opt.eventTypePrefix;
  const image = record.dynamodb.NewImage || record.dynamodb.OldImage;
  const discriminator = image[opt.discriminatorFn] || image[opt.skFn];
  return discriminator.S.toLowerCase();
};
const calculateEventTypeSuffix = record => {
  const suffix = {
    INSERT: 'created',
    MODIFY: 'updated',
    REMOVE: 'deleted'
  }[record.eventName];
  if (suffix !== 'deleted') {
    const {
      NewImage,
      OldImage
    } = record.dynamodb;
    if (NewImage && NewImage.deleted || OldImage && OldImage.deleted) {
      if (NewImage && NewImage.deleted && NewImage.deleted.BOOL) {
        return 'deleted';
      } else {
        /* istanbul ignore else */
        if (OldImage && OldImage.deleted && OldImage.deleted.BOOL) {
          // eslint-disable-line no-lonely-if
          return 'undeleted';
        }
      }
    }
  }
  return suffix;
};

//--------------------------------------------
// global table support - version: 2017.11.29
//--------------------------------------------
// or use the following:
//
// filterPatterns:
//   - eventName: [ INSERT, MODIFY ]
//     dynamodb:
//       NewImage:
//         awsregion:
//           S:
//             - ${opt:region}
//   - eventName: [ REMOVE ]
//     dynamodb:
//       OldImage:
//         awsregion:
//           S:
//             - ${opt:region}

const outReplicas = ignoreReplicas => record => {
  if (!ignoreReplicas) return true;
  const image = record.dynamodb.NewImage || record.dynamodb.OldImage;

  // is this a global table event
  // v2
  if (image.awsregion) {
    // only process events from the current region
    return image.awsregion.S === process.env.AWS_REGION;
  }

  // v1
  /* istanbul ignore next */
  if (image['aws:rep:updateregion']) {
    // only process events from the current region
    return image['aws:rep:updateregion'].S === process.env.AWS_REGION;
  }
  return true;
};

// dynamodb stream emits extra events as it adorns the 'aws:rep' global table metadata
exports.outReplicas = outReplicas;
const outGlobalTableExtraModify = record => {
  const {
    NewImage,
    OldImage
  } = record.dynamodb;

  // v1/v2 transition
  if (NewImage && NewImage.awsregion && OldImage && !OldImage.awsregion && !OldImage['aws:rep:updateregion']) {
    // skip
    return false;
  }

  // v1
  /* istanbul ignore next */
  if (NewImage && NewImage['aws:rep:updateregion'] && OldImage && !OldImage['aws:rep:updateregion']) {
    // skip
    return false;
  }
  return true;
};

//--------------------------------------------
// ttl support or use filterPatterns
//--------------------------------------------
exports.outGlobalTableExtraModify = outGlobalTableExtraModify;
const outTtlExpiredEvents = ignoreTtlExpiredEvents => record => {
  // this is not a REMOVE event
  if (record.eventName !== 'REMOVE') return true;
  const {
    OldImage
  } = record.dynamodb;

  // this record does not have ttl
  if (!OldImage.ttl || !OldImage.timestamp) return true;

  // ttl has not expired
  if (Number(OldImage.ttl.N) * 1000 > Number(OldImage.timestamp.N)) return true;

  // this is a ttl expired event
  // should we ignore it
  /* istanbul ignore else */
  if (ignoreTtlExpiredEvents) {
    return false;
  } else {
    return true;
  }
};

// test helper
exports.outTtlExpiredEvents = outTtlExpiredEvents;
const toDynamodbRecords = (events, {
  removeUndefinedValues = true
} = {}) => ({
  Records: events.map((e, i) => ({
    eventID: `${i}`,
    eventName: !e.oldImage ? 'INSERT' : !e.newImage ? 'REMOVE' : 'MODIFY',
    // eslint-disable-line no-nested-ternary
    // eventVersion: '1.0',
    eventSource: 'aws:dynamodb',
    awsRegion: e.newImage?.awsregion || process.env.AWS_REGION || /* istanbul ignore next */'us-west-2',
    dynamodb: {
      ApproximateCreationDateTime: e.timestamp,
      Keys: e.keys ? (0, _utilDynamodb.marshall)(e.keys, {
        removeUndefinedValues
      }) : /* istanbul ignore next */undefined,
      NewImage: e.newImage ? (0, _utilDynamodb.marshall)(e.newImage, {
        removeUndefinedValues
      }) : undefined,
      OldImage: e.oldImage ? (0, _utilDynamodb.marshall)(e.oldImage, {
        removeUndefinedValues
      }) : undefined,
      SequenceNumber: `${i}`,
      // SizeBytes: 59,
      StreamViewType: 'NEW_AND_OLD_IMAGES'
    }
    // eventSourceARN: 'arn:aws:dynamodb:us-west-2:123456789012:table/myservice-entities/stream/2016-11-16T20:42:48.104',
  }))
});
exports.toDynamodbRecords = toDynamodbRecords;
const UNKNOWN_DYNAMODB_EVENT_TYPE = exports.UNKNOWN_DYNAMODB_EVENT_TYPE = toDynamodbRecords([{
  timestamp: 0,
  keys: {
    pk: '0',
    sk: '0'
  },
  newImage: {
    discriminator: 'unknown'
  }
}]);